/* m_fixts.c
 * Fixes a TS 0 channel by changing the TS 0 to the current TS.
 * Copyright (C) 2003 by MusIRCd Development.
 *
 * $Id: m_fixts.c,v 1.2 2003/10/19 00:14:16 musirc Exp $
 */

#include "stdinc.h"
#include "handlers.h"
#include "ircd.h"
#include "client.h"
#include "send.h"
#include "numeric.h"
#include "hash.h"
#include "modules.h"
#include "s_serv.h"

static void mo_fixts(struct Client *, struct Client *, int, char **);
static void ms_fixts(struct Client *, struct Client *, int, char **);
static void set_fixts_capab(void);
static void unset_fixts_capab(void);

#define CAP_FIXTS 0x00200000

struct Message fixts_msgtab = {
  "FIXTS", 0, 0, 2, 0, MFLG_SLOW, 0,
  {m_unregistered, m_not_oper, ms_fixts, mo_fixts, m_ignore}
};

#ifndef STATIC_MODULES
void
_modinit(void)
{
  mod_add_cmd(&fixts_msgtab);
  set_fixts_capab();
  add_capability("FIXTS", CAP_FIXTS, 1);
}

void
_moddeinit(void)
{
  mod_del_cmd(&fixts_msgtab);
  unset_fixts_capab();
  delete_capability("FIXTS");
}

const char *_version = "$Revision: 1.2 $";
#endif

/* mo_fixts()
 *      parv[0] = sender prefix
 *      parv[1] = channel
 */
static void
mo_fixts(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
{
  struct Channel *chptr;
  int currentts = CurrentTime;

  if (parc < 2)
  {
    sendto_one(source_p, form_str(ERR_NEEDMOREPARAMS),
               me.name, source_p->name, "FIXTS");
    return;
  }

  if (!IsAdmin(source_p))
  {
    sendto_one(source_p, form_str(ERR_NOPRIVILEGES),
               me.name, source_p->name);
    return;
  }

  if ((chptr = hash_find_channel(parv[1])) == NULL)
  {
    sendto_one(source_p, form_str(ERR_NOSUCHCHANNEL),
	       me.name, source_p->name, parv[1]);
    return;
  }

  if (chptr->channelts == 0)
  {
    chptr->channelts = currentts;
    sendto_channel_local(ALL_MEMBERS, chptr,
                         ":%s NOTICE %s :*** Notice -- TS for %s changed from 0 to %lu",
                         me.name, chptr->chname, chptr->chname, (unsigned long)currentts);
    sendto_realops_flags(UMODE_ALL, L_ALL,
                         "%s changing TS on %s from 0 to %lu",
                         source_p->name, chptr->chname, (unsigned long)currentts);
    if (HasID(source_p))
    {
      sendto_server(NULL, source_p, NULL, CAP_FIXTS|CAP_TS6, NOCAPS, LL_ICLIENT,
                    ":%s FIXTS %s", source_p->id, chptr->chname);
      sendto_server(NULL, source_p, NULL, CAP_FIXTS, CAP_TS6, LL_ICLIENT,
                    ":%s FIXTS %s", source_p->name, chptr->chname);
    }
    else
      sendto_server(NULL, source_p, NULL, CAP_FIXTS, NOCAPS, LL_ICLIENT,
                    ":%s FIXTS %s", source_p->name, chptr->chname);

  }
  else
    sendto_one(source_p, ":%s NOTICE %s :*** Notice -- %s does not have a TS of 0",
               me.name, source_p->name, chptr->chname);
}

/* ms_fixts()
 *      parv[0] = sender prefix
 *      parv[1] = channel
 */
static void
ms_fixts(struct Client *client_p, struct Client *source_p,
         int parc, char *parv[])
{
  struct Channel *chptr;
  int currentts = CurrentTime;

  if (parc != 2)
    return;

  if ((chptr = hash_find_channel(parv[1])) == NULL)
    return;

  if (chptr->channelts == 0)
  {
    chptr->channelts = currentts;
    sendto_channel_local(ALL_MEMBERS, chptr,
                         ":%s NOTICE %s :*** Notice -- TS for %s changed from 0 to %lu",
                         me.name, chptr->chname, chptr->chname, (unsigned long)currentts);
    sendto_realops_flags(UMODE_ALL, L_ALL,
                         "%s changing TS on %s from 0 to %lu",
                         source_p->name, chptr->chname, (unsigned long)currentts);
  }
}

static void
set_fixts_capab(void)
{
  default_server_capabs |= CAP_FIXTS;
}

static void
unset_fixts_capab(void)
{
  default_server_capabs &= ~CAP_FIXTS;
}

